# This script refers to the tasks described in "C.1 Activities", activities 1-3.

library(dplyr)
library(lavaan)
library(eRm)
library(TAM)
library(mirt)
library(directlabels)
library(ltm)

df <- read.csv("BLOT.csv", header = FALSE, sep = ";")

colnames(df)[1] <- c("id")
colnames(df)[2:49] <- paste("blot", (1:48), sep="_")
names(df)
blot_items <- dplyr::select(df, blot_1:blot_35) # Selecting BLOT items

# lavaan: RM as a CFA model. By using 'a' as the same label for all item slopes, slopes will be constrained to be equal across items: 
rasch_model <- 'eta =~ a*blot_1 + a*blot_2 + a*blot_3 + a*blot_4 + a*blot_5 + a*blot_6 + a*blot_7 + a*blot_8 + a*blot_9 + a*blot_10 + a*blot_11 + a*blot_12 + a*blot_13 + a*blot_14 + a*blot_15 + a*blot_16 + a*blot_17 + a*blot_18 + a*blot_19 + a*blot_20 + a*blot_21 + a*blot_22 + a*blot_23 + a*blot_24 + a*blot_25 + a*blot_26 + a*blot_27 + a*blot_28 + a*blot_29 + a*blot_30 + a*blot_31 + a*blot_32 + a*blot_33 + a*blot_34 + a*blot_35
'

fit_eta_rasch <- lavaan::cfa(rasch_model, std.lv = TRUE, data = blot_items, 
                             ordered = TRUE)

summary(fit_eta_rasch, fit.measures = TRUE)

two_pl_blot <- 'eta =~ blot_1 + blot_2 + blot_3 + blot_4 + blot_5 + blot_6 + blot_7 + blot_8 + blot_9 + blot_10 + blot_11 + blot_12 + blot_13 + blot_14 + blot_15 + blot_16 + blot_17 + blot_18 + blot_19 + blot_20 + blot_21 + blot_22 + blot_23 + blot_24 + blot_25 + blot_26 + blot_27 + blot_28 + blot_29 + blot_30 + blot_31 + blot_32 + blot_33 + blot_34 + blot_35
' 

# Estimating model parameters using lavaan:
fit_eta_2pl <- lavaan::cfa(two_pl_blot, std.lv = TRUE, data = blot_items, 
ordered = TRUE)
summary(fit_eta_2pl, fit.measures = TRUE)

# Chi-square difference test: 2pl vs. Rasch model comparison
anova(fit_eta_rasch, fit_eta_2pl)


# TAM:
fit_rasch_tam <- TAM::tam.mml(blot_items)
summary(fit_rasch_tam)
summary(tam.modelfit(fit_rasch_tam)$aQ3.matr) # Show largest adjusted residual correlations if needed
fit_2pl_tam <- tam.mml.2pl(blot_items)
summary(fit_2pl_tam)
IRT.compareModels(fit_rasch_tam, fit_2pl_tam)

# Mean and SD of item slopes (IRT metric):
round(mean(fit_2pl_tam$item_irt$alpha), 2)
round(sd(fit_2pl_tam$item_irt$alpha), 2)

# Plot ICC curves of the 2pl and Rasch model using the "mirt" package
colnames(blot_items)[1:35] <- paste("#",(1:35), sep = "") # New names for better labels in plot. Notice that Figure C.2 in ARM4 labels only selected items
two.pl.mirt <- mirt(blot_items, 1, technical = list(NCYCLES = 2000))
bwtheme <- standard.theme("pdf", color=FALSE) # Sets color scale to black-white
plt <- plot(two.pl.mirt, type = 'trace', auto.key = FALSE, facet_items = FALSE, par.settings = bwtheme, main = "")
plt
direct.label(plt, list(cex=.5,'angled.boxes')) # Adding item labels.


# Andersen likleihood ratio test with the eRm package:
rm <- RM(blot_items) # Estimates Rasch model parameters
(lrtest <- LRtest(rm)) # Andersen LRT results
(item_fit <- eRm::Waldtest(rm)) # Which item parameters are significantly different between the score groups?

# Rasch's graphical model check:
plotGOF(lrtest, col = "black", tlab = "identify", xlim = c(-5, 5), ylim = c(-2, 3), pos = 1) # Important notice from the helpfile (?plotGOF): "Initially no labels are printed, after clicking close to an item point the corresponding label is added. The identification process is terminated by clicking the second button and selecting 'Stop' from the menu, or from the 'Stop' menu on the graphics window."
